# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: deployment.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
import bentoml.proto.status_pb2 as status__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='deployment.proto',
  package='bentoml',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x10\x64\x65ployment.proto\x12\x07\x62\x65ntoml\x1a\x19google/protobuf/any.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x0cstatus.proto\"\xc9\t\n\x0e\x44\x65ploymentSpec\x12\x12\n\nbento_name\x18\x01 \x01(\t\x12\x15\n\rbento_version\x18\x02 \x01(\t\x12<\n\x08operator\x18\x03 \x01(\x0e\x32*.bentoml.DeploymentSpec.DeploymentOperator\x12N\n\x16\x63ustom_operator_config\x18\x04 \x01(\x0b\x32,.bentoml.DeploymentSpec.CustomOperatorConfigH\x00\x12T\n\x19sagemaker_operator_config\x18\x05 \x01(\x0b\x32/.bentoml.DeploymentSpec.SageMakerOperatorConfigH\x00\x12U\n\x1a\x61ws_lambda_operator_config\x18\x06 \x01(\x0b\x32/.bentoml.DeploymentSpec.AwsLambdaOperatorConfigH\x00\x12Y\n\x1cgcp_function_operator_config\x18\x07 \x01(\x0b\x32\x31.bentoml.DeploymentSpec.GcpFunctionOperatorConfigH\x00\x12V\n\x1akubernetes_operator_config\x18\x08 \x01(\x0b\x32\x30.bentoml.DeploymentSpec.KubernetesOperatorConfigH\x00\x1a\xb3\x01\n\x14\x43ustomOperatorConfig\x12\x0c\n\x04name\x18\x01 \x01(\t\x12H\n\x06\x63onfig\x18\x02 \x03(\x0b\x32\x38.bentoml.DeploymentSpec.CustomOperatorConfig.ConfigEntry\x1a\x43\n\x0b\x43onfigEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any:\x02\x38\x01\x1aj\n\x17SageMakerOperatorConfig\x12\x0e\n\x06region\x18\x01 \x01(\t\x12\x15\n\rinstance_type\x18\x02 \x01(\t\x12\x16\n\x0einstance_count\x18\x03 \x01(\x05\x12\x10\n\x08\x61pi_name\x18\x04 \x01(\t\x1a;\n\x17\x41wsLambdaOperatorConfig\x12\x0e\n\x06region\x18\x01 \x01(\t\x12\x10\n\x08\x61pi_name\x18\x02 \x01(\t\x1a=\n\x19GcpFunctionOperatorConfig\x12\x0e\n\x06region\x18\x01 \x01(\t\x12\x10\n\x08\x61pi_name\x18\x02 \x01(\t\x1ap\n\x18KubernetesOperatorConfig\x12\x16\n\x0ekube_namespace\x18\x01 \x01(\t\x12\x10\n\x08replicas\x18\x02 \x01(\x05\x12\x14\n\x0cservice_name\x18\x03 \x01(\t\x12\x14\n\x0cservice_type\x18\x04 \x01(\t\"p\n\x12\x44\x65ploymentOperator\x12\t\n\x05UNSET\x10\x00\x12\n\n\x06\x43USTOM\x10\x01\x12\x11\n\rAWS_SAGEMAKER\x10\x02\x12\x0e\n\nAWS_LAMBDA\x10\x03\x12\x10\n\x0cGCP_FUNCTION\x10\x04\x12\x0e\n\nKUBERNETES\x10\x05\x42\x1c\n\x1a\x64\x65ployment_operator_config\"\xa9\x02\n\x0f\x44\x65ploymentState\x12-\n\x05state\x18\x01 \x01(\x0e\x32\x1e.bentoml.DeploymentState.State\x12\x15\n\rerror_message\x18\x02 \x01(\t\x12\x11\n\tinfo_json\x18\x03 \x01(\t\x12-\n\ttimestamp\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\x8d\x01\n\x05State\x12\x0b\n\x07PENDING\x10\x00\x12\x0b\n\x07RUNNING\x10\x01\x12\r\n\tSUCCEEDED\x10\x02\x12\n\n\x06\x46\x41ILED\x10\x03\x12\x0b\n\x07UNKNOWN\x10\x04\x12\r\n\tCOMPLETED\x10\x05\x12\x17\n\x13\x43RASH_LOOP_BACK_OFF\x10\x06\x12\t\n\x05\x45RROR\x10\x07\x12\x0f\n\x0bINACTIVATED\x10\x08\"\xb1\x03\n\nDeployment\x12\x11\n\tnamespace\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12%\n\x04spec\x18\x03 \x01(\x0b\x32\x17.bentoml.DeploymentSpec\x12\'\n\x05state\x18\x04 \x01(\x0b\x32\x18.bentoml.DeploymentState\x12\x39\n\x0b\x61nnotations\x18\x05 \x03(\x0b\x32$.bentoml.Deployment.AnnotationsEntry\x12/\n\x06labels\x18\x06 \x03(\x0b\x32\x1f.bentoml.Deployment.LabelsEntry\x12.\n\ncreated_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x33\n\x0flast_updated_at\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\";\n\x10\x44\x65ploymentStatus\x12\'\n\x05state\x18\x01 \x01(\x0b\x32\x18.bentoml.DeploymentState\"A\n\x16\x41pplyDeploymentRequest\x12\'\n\ndeployment\x18\x01 \x01(\x0b\x32\x13.bentoml.Deployment\"c\n\x17\x41pplyDeploymentResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\x12\'\n\ndeployment\x18\x02 \x01(\x0b\x32\x13.bentoml.Deployment\"[\n\x17\x44\x65leteDeploymentRequest\x12\x17\n\x0f\x64\x65ployment_name\x18\x01 \x01(\t\x12\x11\n\tnamespace\x18\x02 \x01(\t\x12\x14\n\x0c\x66orce_delete\x18\x03 \x01(\x08\";\n\x18\x44\x65leteDeploymentResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\"B\n\x14GetDeploymentRequest\x12\x17\n\x0f\x64\x65ployment_name\x18\x01 \x01(\t\x12\x11\n\tnamespace\x18\x02 \x01(\t\"a\n\x15GetDeploymentResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\x12\'\n\ndeployment\x18\x02 \x01(\x0b\x32\x13.bentoml.Deployment\"G\n\x19\x44\x65scribeDeploymentRequest\x12\x17\n\x0f\x64\x65ployment_name\x18\x01 \x01(\t\x12\x11\n\tnamespace\x18\x02 \x01(\t\"f\n\x1a\x44\x65scribeDeploymentResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\x12\'\n\x05state\x18\x02 \x01(\x0b\x32\x18.bentoml.DeploymentState\"\xc6\x01\n\x16ListDeploymentsRequest\x12\x11\n\tnamespace\x18\x01 \x01(\t\x12\x0e\n\x06offset\x18\x02 \x01(\x05\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\x0e\n\x06\x66ilter\x18\x04 \x01(\t\x12;\n\x06labels\x18\x05 \x03(\x0b\x32+.bentoml.ListDeploymentsRequest.LabelsEntry\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"d\n\x17ListDeploymentsResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\x12(\n\x0b\x64\x65ployments\x18\x02 \x03(\x0b\x32\x13.bentoml.Deploymentb\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_any__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,status__pb2.DESCRIPTOR,])



_DEPLOYMENTSPEC_DEPLOYMENTOPERATOR = _descriptor.EnumDescriptor(
  name='DeploymentOperator',
  full_name='bentoml.DeploymentSpec.DeploymentOperator',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNSET', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CUSTOM', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='AWS_SAGEMAKER', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='AWS_LAMBDA', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GCP_FUNCTION', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='KUBERNETES', index=5, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1187,
  serialized_end=1299,
)
_sym_db.RegisterEnumDescriptor(_DEPLOYMENTSPEC_DEPLOYMENTOPERATOR)

_DEPLOYMENTSTATE_STATE = _descriptor.EnumDescriptor(
  name='State',
  full_name='bentoml.DeploymentState.State',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PENDING', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RUNNING', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUCCEEDED', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAILED', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='COMPLETED', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CRASH_LOOP_BACK_OFF', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=7, number=7,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INACTIVATED', index=8, number=8,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1488,
  serialized_end=1629,
)
_sym_db.RegisterEnumDescriptor(_DEPLOYMENTSTATE_STATE)


_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG_CONFIGENTRY = _descriptor.Descriptor(
  name='ConfigEntry',
  full_name='bentoml.DeploymentSpec.CustomOperatorConfig.ConfigEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='bentoml.DeploymentSpec.CustomOperatorConfig.ConfigEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='bentoml.DeploymentSpec.CustomOperatorConfig.ConfigEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=772,
  serialized_end=839,
)

_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG = _descriptor.Descriptor(
  name='CustomOperatorConfig',
  full_name='bentoml.DeploymentSpec.CustomOperatorConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bentoml.DeploymentSpec.CustomOperatorConfig.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='config', full_name='bentoml.DeploymentSpec.CustomOperatorConfig.config', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG_CONFIGENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=660,
  serialized_end=839,
)

_DEPLOYMENTSPEC_SAGEMAKEROPERATORCONFIG = _descriptor.Descriptor(
  name='SageMakerOperatorConfig',
  full_name='bentoml.DeploymentSpec.SageMakerOperatorConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='region', full_name='bentoml.DeploymentSpec.SageMakerOperatorConfig.region', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='instance_type', full_name='bentoml.DeploymentSpec.SageMakerOperatorConfig.instance_type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='instance_count', full_name='bentoml.DeploymentSpec.SageMakerOperatorConfig.instance_count', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='api_name', full_name='bentoml.DeploymentSpec.SageMakerOperatorConfig.api_name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=841,
  serialized_end=947,
)

_DEPLOYMENTSPEC_AWSLAMBDAOPERATORCONFIG = _descriptor.Descriptor(
  name='AwsLambdaOperatorConfig',
  full_name='bentoml.DeploymentSpec.AwsLambdaOperatorConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='region', full_name='bentoml.DeploymentSpec.AwsLambdaOperatorConfig.region', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='api_name', full_name='bentoml.DeploymentSpec.AwsLambdaOperatorConfig.api_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=949,
  serialized_end=1008,
)

_DEPLOYMENTSPEC_GCPFUNCTIONOPERATORCONFIG = _descriptor.Descriptor(
  name='GcpFunctionOperatorConfig',
  full_name='bentoml.DeploymentSpec.GcpFunctionOperatorConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='region', full_name='bentoml.DeploymentSpec.GcpFunctionOperatorConfig.region', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='api_name', full_name='bentoml.DeploymentSpec.GcpFunctionOperatorConfig.api_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1010,
  serialized_end=1071,
)

_DEPLOYMENTSPEC_KUBERNETESOPERATORCONFIG = _descriptor.Descriptor(
  name='KubernetesOperatorConfig',
  full_name='bentoml.DeploymentSpec.KubernetesOperatorConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='kube_namespace', full_name='bentoml.DeploymentSpec.KubernetesOperatorConfig.kube_namespace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='replicas', full_name='bentoml.DeploymentSpec.KubernetesOperatorConfig.replicas', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='service_name', full_name='bentoml.DeploymentSpec.KubernetesOperatorConfig.service_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='service_type', full_name='bentoml.DeploymentSpec.KubernetesOperatorConfig.service_type', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1073,
  serialized_end=1185,
)

_DEPLOYMENTSPEC = _descriptor.Descriptor(
  name='DeploymentSpec',
  full_name='bentoml.DeploymentSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bento_name', full_name='bentoml.DeploymentSpec.bento_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bento_version', full_name='bentoml.DeploymentSpec.bento_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='operator', full_name='bentoml.DeploymentSpec.operator', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='custom_operator_config', full_name='bentoml.DeploymentSpec.custom_operator_config', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sagemaker_operator_config', full_name='bentoml.DeploymentSpec.sagemaker_operator_config', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='aws_lambda_operator_config', full_name='bentoml.DeploymentSpec.aws_lambda_operator_config', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gcp_function_operator_config', full_name='bentoml.DeploymentSpec.gcp_function_operator_config', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='kubernetes_operator_config', full_name='bentoml.DeploymentSpec.kubernetes_operator_config', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG, _DEPLOYMENTSPEC_SAGEMAKEROPERATORCONFIG, _DEPLOYMENTSPEC_AWSLAMBDAOPERATORCONFIG, _DEPLOYMENTSPEC_GCPFUNCTIONOPERATORCONFIG, _DEPLOYMENTSPEC_KUBERNETESOPERATORCONFIG, ],
  enum_types=[
    _DEPLOYMENTSPEC_DEPLOYMENTOPERATOR,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='deployment_operator_config', full_name='bentoml.DeploymentSpec.deployment_operator_config',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=104,
  serialized_end=1329,
)


_DEPLOYMENTSTATE = _descriptor.Descriptor(
  name='DeploymentState',
  full_name='bentoml.DeploymentState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='bentoml.DeploymentState.state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error_message', full_name='bentoml.DeploymentState.error_message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info_json', full_name='bentoml.DeploymentState.info_json', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='bentoml.DeploymentState.timestamp', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _DEPLOYMENTSTATE_STATE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1332,
  serialized_end=1629,
)


_DEPLOYMENT_ANNOTATIONSENTRY = _descriptor.Descriptor(
  name='AnnotationsEntry',
  full_name='bentoml.Deployment.AnnotationsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='bentoml.Deployment.AnnotationsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='bentoml.Deployment.AnnotationsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1968,
  serialized_end=2018,
)

_DEPLOYMENT_LABELSENTRY = _descriptor.Descriptor(
  name='LabelsEntry',
  full_name='bentoml.Deployment.LabelsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='bentoml.Deployment.LabelsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='bentoml.Deployment.LabelsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2020,
  serialized_end=2065,
)

_DEPLOYMENT = _descriptor.Descriptor(
  name='Deployment',
  full_name='bentoml.Deployment',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespace', full_name='bentoml.Deployment.namespace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='bentoml.Deployment.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='spec', full_name='bentoml.Deployment.spec', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='state', full_name='bentoml.Deployment.state', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='annotations', full_name='bentoml.Deployment.annotations', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='labels', full_name='bentoml.Deployment.labels', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='bentoml.Deployment.created_at', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='last_updated_at', full_name='bentoml.Deployment.last_updated_at', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_DEPLOYMENT_ANNOTATIONSENTRY, _DEPLOYMENT_LABELSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1632,
  serialized_end=2065,
)


_DEPLOYMENTSTATUS = _descriptor.Descriptor(
  name='DeploymentStatus',
  full_name='bentoml.DeploymentStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='bentoml.DeploymentStatus.state', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2067,
  serialized_end=2126,
)


_APPLYDEPLOYMENTREQUEST = _descriptor.Descriptor(
  name='ApplyDeploymentRequest',
  full_name='bentoml.ApplyDeploymentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='deployment', full_name='bentoml.ApplyDeploymentRequest.deployment', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2128,
  serialized_end=2193,
)


_APPLYDEPLOYMENTRESPONSE = _descriptor.Descriptor(
  name='ApplyDeploymentResponse',
  full_name='bentoml.ApplyDeploymentResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.ApplyDeploymentResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deployment', full_name='bentoml.ApplyDeploymentResponse.deployment', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2195,
  serialized_end=2294,
)


_DELETEDEPLOYMENTREQUEST = _descriptor.Descriptor(
  name='DeleteDeploymentRequest',
  full_name='bentoml.DeleteDeploymentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='deployment_name', full_name='bentoml.DeleteDeploymentRequest.deployment_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespace', full_name='bentoml.DeleteDeploymentRequest.namespace', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='force_delete', full_name='bentoml.DeleteDeploymentRequest.force_delete', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2296,
  serialized_end=2387,
)


_DELETEDEPLOYMENTRESPONSE = _descriptor.Descriptor(
  name='DeleteDeploymentResponse',
  full_name='bentoml.DeleteDeploymentResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.DeleteDeploymentResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2389,
  serialized_end=2448,
)


_GETDEPLOYMENTREQUEST = _descriptor.Descriptor(
  name='GetDeploymentRequest',
  full_name='bentoml.GetDeploymentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='deployment_name', full_name='bentoml.GetDeploymentRequest.deployment_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespace', full_name='bentoml.GetDeploymentRequest.namespace', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2450,
  serialized_end=2516,
)


_GETDEPLOYMENTRESPONSE = _descriptor.Descriptor(
  name='GetDeploymentResponse',
  full_name='bentoml.GetDeploymentResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.GetDeploymentResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deployment', full_name='bentoml.GetDeploymentResponse.deployment', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2518,
  serialized_end=2615,
)


_DESCRIBEDEPLOYMENTREQUEST = _descriptor.Descriptor(
  name='DescribeDeploymentRequest',
  full_name='bentoml.DescribeDeploymentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='deployment_name', full_name='bentoml.DescribeDeploymentRequest.deployment_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespace', full_name='bentoml.DescribeDeploymentRequest.namespace', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2617,
  serialized_end=2688,
)


_DESCRIBEDEPLOYMENTRESPONSE = _descriptor.Descriptor(
  name='DescribeDeploymentResponse',
  full_name='bentoml.DescribeDeploymentResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.DescribeDeploymentResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='state', full_name='bentoml.DescribeDeploymentResponse.state', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2690,
  serialized_end=2792,
)


_LISTDEPLOYMENTSREQUEST_LABELSENTRY = _descriptor.Descriptor(
  name='LabelsEntry',
  full_name='bentoml.ListDeploymentsRequest.LabelsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='bentoml.ListDeploymentsRequest.LabelsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='bentoml.ListDeploymentsRequest.LabelsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2020,
  serialized_end=2065,
)

_LISTDEPLOYMENTSREQUEST = _descriptor.Descriptor(
  name='ListDeploymentsRequest',
  full_name='bentoml.ListDeploymentsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespace', full_name='bentoml.ListDeploymentsRequest.namespace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='offset', full_name='bentoml.ListDeploymentsRequest.offset', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='limit', full_name='bentoml.ListDeploymentsRequest.limit', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='filter', full_name='bentoml.ListDeploymentsRequest.filter', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='labels', full_name='bentoml.ListDeploymentsRequest.labels', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_LISTDEPLOYMENTSREQUEST_LABELSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2795,
  serialized_end=2993,
)


_LISTDEPLOYMENTSRESPONSE = _descriptor.Descriptor(
  name='ListDeploymentsResponse',
  full_name='bentoml.ListDeploymentsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.ListDeploymentsResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deployments', full_name='bentoml.ListDeploymentsResponse.deployments', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2995,
  serialized_end=3095,
)

_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG_CONFIGENTRY.fields_by_name['value'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG_CONFIGENTRY.containing_type = _DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG
_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG.fields_by_name['config'].message_type = _DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG_CONFIGENTRY
_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG.containing_type = _DEPLOYMENTSPEC
_DEPLOYMENTSPEC_SAGEMAKEROPERATORCONFIG.containing_type = _DEPLOYMENTSPEC
_DEPLOYMENTSPEC_AWSLAMBDAOPERATORCONFIG.containing_type = _DEPLOYMENTSPEC
_DEPLOYMENTSPEC_GCPFUNCTIONOPERATORCONFIG.containing_type = _DEPLOYMENTSPEC
_DEPLOYMENTSPEC_KUBERNETESOPERATORCONFIG.containing_type = _DEPLOYMENTSPEC
_DEPLOYMENTSPEC.fields_by_name['operator'].enum_type = _DEPLOYMENTSPEC_DEPLOYMENTOPERATOR
_DEPLOYMENTSPEC.fields_by_name['custom_operator_config'].message_type = _DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG
_DEPLOYMENTSPEC.fields_by_name['sagemaker_operator_config'].message_type = _DEPLOYMENTSPEC_SAGEMAKEROPERATORCONFIG
_DEPLOYMENTSPEC.fields_by_name['aws_lambda_operator_config'].message_type = _DEPLOYMENTSPEC_AWSLAMBDAOPERATORCONFIG
_DEPLOYMENTSPEC.fields_by_name['gcp_function_operator_config'].message_type = _DEPLOYMENTSPEC_GCPFUNCTIONOPERATORCONFIG
_DEPLOYMENTSPEC.fields_by_name['kubernetes_operator_config'].message_type = _DEPLOYMENTSPEC_KUBERNETESOPERATORCONFIG
_DEPLOYMENTSPEC_DEPLOYMENTOPERATOR.containing_type = _DEPLOYMENTSPEC
_DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config'].fields.append(
  _DEPLOYMENTSPEC.fields_by_name['custom_operator_config'])
_DEPLOYMENTSPEC.fields_by_name['custom_operator_config'].containing_oneof = _DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config']
_DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config'].fields.append(
  _DEPLOYMENTSPEC.fields_by_name['sagemaker_operator_config'])
_DEPLOYMENTSPEC.fields_by_name['sagemaker_operator_config'].containing_oneof = _DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config']
_DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config'].fields.append(
  _DEPLOYMENTSPEC.fields_by_name['aws_lambda_operator_config'])
_DEPLOYMENTSPEC.fields_by_name['aws_lambda_operator_config'].containing_oneof = _DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config']
_DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config'].fields.append(
  _DEPLOYMENTSPEC.fields_by_name['gcp_function_operator_config'])
_DEPLOYMENTSPEC.fields_by_name['gcp_function_operator_config'].containing_oneof = _DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config']
_DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config'].fields.append(
  _DEPLOYMENTSPEC.fields_by_name['kubernetes_operator_config'])
_DEPLOYMENTSPEC.fields_by_name['kubernetes_operator_config'].containing_oneof = _DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config']
_DEPLOYMENTSTATE.fields_by_name['state'].enum_type = _DEPLOYMENTSTATE_STATE
_DEPLOYMENTSTATE.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_DEPLOYMENTSTATE_STATE.containing_type = _DEPLOYMENTSTATE
_DEPLOYMENT_ANNOTATIONSENTRY.containing_type = _DEPLOYMENT
_DEPLOYMENT_LABELSENTRY.containing_type = _DEPLOYMENT
_DEPLOYMENT.fields_by_name['spec'].message_type = _DEPLOYMENTSPEC
_DEPLOYMENT.fields_by_name['state'].message_type = _DEPLOYMENTSTATE
_DEPLOYMENT.fields_by_name['annotations'].message_type = _DEPLOYMENT_ANNOTATIONSENTRY
_DEPLOYMENT.fields_by_name['labels'].message_type = _DEPLOYMENT_LABELSENTRY
_DEPLOYMENT.fields_by_name['created_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_DEPLOYMENT.fields_by_name['last_updated_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_DEPLOYMENTSTATUS.fields_by_name['state'].message_type = _DEPLOYMENTSTATE
_APPLYDEPLOYMENTREQUEST.fields_by_name['deployment'].message_type = _DEPLOYMENT
_APPLYDEPLOYMENTRESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_APPLYDEPLOYMENTRESPONSE.fields_by_name['deployment'].message_type = _DEPLOYMENT
_DELETEDEPLOYMENTRESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_GETDEPLOYMENTRESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_GETDEPLOYMENTRESPONSE.fields_by_name['deployment'].message_type = _DEPLOYMENT
_DESCRIBEDEPLOYMENTRESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_DESCRIBEDEPLOYMENTRESPONSE.fields_by_name['state'].message_type = _DEPLOYMENTSTATE
_LISTDEPLOYMENTSREQUEST_LABELSENTRY.containing_type = _LISTDEPLOYMENTSREQUEST
_LISTDEPLOYMENTSREQUEST.fields_by_name['labels'].message_type = _LISTDEPLOYMENTSREQUEST_LABELSENTRY
_LISTDEPLOYMENTSRESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_LISTDEPLOYMENTSRESPONSE.fields_by_name['deployments'].message_type = _DEPLOYMENT
DESCRIPTOR.message_types_by_name['DeploymentSpec'] = _DEPLOYMENTSPEC
DESCRIPTOR.message_types_by_name['DeploymentState'] = _DEPLOYMENTSTATE
DESCRIPTOR.message_types_by_name['Deployment'] = _DEPLOYMENT
DESCRIPTOR.message_types_by_name['DeploymentStatus'] = _DEPLOYMENTSTATUS
DESCRIPTOR.message_types_by_name['ApplyDeploymentRequest'] = _APPLYDEPLOYMENTREQUEST
DESCRIPTOR.message_types_by_name['ApplyDeploymentResponse'] = _APPLYDEPLOYMENTRESPONSE
DESCRIPTOR.message_types_by_name['DeleteDeploymentRequest'] = _DELETEDEPLOYMENTREQUEST
DESCRIPTOR.message_types_by_name['DeleteDeploymentResponse'] = _DELETEDEPLOYMENTRESPONSE
DESCRIPTOR.message_types_by_name['GetDeploymentRequest'] = _GETDEPLOYMENTREQUEST
DESCRIPTOR.message_types_by_name['GetDeploymentResponse'] = _GETDEPLOYMENTRESPONSE
DESCRIPTOR.message_types_by_name['DescribeDeploymentRequest'] = _DESCRIBEDEPLOYMENTREQUEST
DESCRIPTOR.message_types_by_name['DescribeDeploymentResponse'] = _DESCRIBEDEPLOYMENTRESPONSE
DESCRIPTOR.message_types_by_name['ListDeploymentsRequest'] = _LISTDEPLOYMENTSREQUEST
DESCRIPTOR.message_types_by_name['ListDeploymentsResponse'] = _LISTDEPLOYMENTSRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DeploymentSpec = _reflection.GeneratedProtocolMessageType('DeploymentSpec', (_message.Message,), {

  'CustomOperatorConfig' : _reflection.GeneratedProtocolMessageType('CustomOperatorConfig', (_message.Message,), {

    'ConfigEntry' : _reflection.GeneratedProtocolMessageType('ConfigEntry', (_message.Message,), {
      'DESCRIPTOR' : _DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG_CONFIGENTRY,
      '__module__' : 'deployment_pb2'
      # @@protoc_insertion_point(class_scope:bentoml.DeploymentSpec.CustomOperatorConfig.ConfigEntry)
      })
    ,
    'DESCRIPTOR' : _DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG,
    '__module__' : 'deployment_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.DeploymentSpec.CustomOperatorConfig)
    })
  ,

  'SageMakerOperatorConfig' : _reflection.GeneratedProtocolMessageType('SageMakerOperatorConfig', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYMENTSPEC_SAGEMAKEROPERATORCONFIG,
    '__module__' : 'deployment_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.DeploymentSpec.SageMakerOperatorConfig)
    })
  ,

  'AwsLambdaOperatorConfig' : _reflection.GeneratedProtocolMessageType('AwsLambdaOperatorConfig', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYMENTSPEC_AWSLAMBDAOPERATORCONFIG,
    '__module__' : 'deployment_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.DeploymentSpec.AwsLambdaOperatorConfig)
    })
  ,

  'GcpFunctionOperatorConfig' : _reflection.GeneratedProtocolMessageType('GcpFunctionOperatorConfig', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYMENTSPEC_GCPFUNCTIONOPERATORCONFIG,
    '__module__' : 'deployment_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.DeploymentSpec.GcpFunctionOperatorConfig)
    })
  ,

  'KubernetesOperatorConfig' : _reflection.GeneratedProtocolMessageType('KubernetesOperatorConfig', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYMENTSPEC_KUBERNETESOPERATORCONFIG,
    '__module__' : 'deployment_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.DeploymentSpec.KubernetesOperatorConfig)
    })
  ,
  'DESCRIPTOR' : _DEPLOYMENTSPEC,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DeploymentSpec)
  })
_sym_db.RegisterMessage(DeploymentSpec)
_sym_db.RegisterMessage(DeploymentSpec.CustomOperatorConfig)
_sym_db.RegisterMessage(DeploymentSpec.CustomOperatorConfig.ConfigEntry)
_sym_db.RegisterMessage(DeploymentSpec.SageMakerOperatorConfig)
_sym_db.RegisterMessage(DeploymentSpec.AwsLambdaOperatorConfig)
_sym_db.RegisterMessage(DeploymentSpec.GcpFunctionOperatorConfig)
_sym_db.RegisterMessage(DeploymentSpec.KubernetesOperatorConfig)

DeploymentState = _reflection.GeneratedProtocolMessageType('DeploymentState', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYMENTSTATE,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DeploymentState)
  })
_sym_db.RegisterMessage(DeploymentState)

Deployment = _reflection.GeneratedProtocolMessageType('Deployment', (_message.Message,), {

  'AnnotationsEntry' : _reflection.GeneratedProtocolMessageType('AnnotationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYMENT_ANNOTATIONSENTRY,
    '__module__' : 'deployment_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.Deployment.AnnotationsEntry)
    })
  ,

  'LabelsEntry' : _reflection.GeneratedProtocolMessageType('LabelsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYMENT_LABELSENTRY,
    '__module__' : 'deployment_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.Deployment.LabelsEntry)
    })
  ,
  'DESCRIPTOR' : _DEPLOYMENT,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.Deployment)
  })
_sym_db.RegisterMessage(Deployment)
_sym_db.RegisterMessage(Deployment.AnnotationsEntry)
_sym_db.RegisterMessage(Deployment.LabelsEntry)

DeploymentStatus = _reflection.GeneratedProtocolMessageType('DeploymentStatus', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYMENTSTATUS,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DeploymentStatus)
  })
_sym_db.RegisterMessage(DeploymentStatus)

ApplyDeploymentRequest = _reflection.GeneratedProtocolMessageType('ApplyDeploymentRequest', (_message.Message,), {
  'DESCRIPTOR' : _APPLYDEPLOYMENTREQUEST,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.ApplyDeploymentRequest)
  })
_sym_db.RegisterMessage(ApplyDeploymentRequest)

ApplyDeploymentResponse = _reflection.GeneratedProtocolMessageType('ApplyDeploymentResponse', (_message.Message,), {
  'DESCRIPTOR' : _APPLYDEPLOYMENTRESPONSE,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.ApplyDeploymentResponse)
  })
_sym_db.RegisterMessage(ApplyDeploymentResponse)

DeleteDeploymentRequest = _reflection.GeneratedProtocolMessageType('DeleteDeploymentRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEDEPLOYMENTREQUEST,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DeleteDeploymentRequest)
  })
_sym_db.RegisterMessage(DeleteDeploymentRequest)

DeleteDeploymentResponse = _reflection.GeneratedProtocolMessageType('DeleteDeploymentResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEDEPLOYMENTRESPONSE,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DeleteDeploymentResponse)
  })
_sym_db.RegisterMessage(DeleteDeploymentResponse)

GetDeploymentRequest = _reflection.GeneratedProtocolMessageType('GetDeploymentRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETDEPLOYMENTREQUEST,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.GetDeploymentRequest)
  })
_sym_db.RegisterMessage(GetDeploymentRequest)

GetDeploymentResponse = _reflection.GeneratedProtocolMessageType('GetDeploymentResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETDEPLOYMENTRESPONSE,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.GetDeploymentResponse)
  })
_sym_db.RegisterMessage(GetDeploymentResponse)

DescribeDeploymentRequest = _reflection.GeneratedProtocolMessageType('DescribeDeploymentRequest', (_message.Message,), {
  'DESCRIPTOR' : _DESCRIBEDEPLOYMENTREQUEST,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DescribeDeploymentRequest)
  })
_sym_db.RegisterMessage(DescribeDeploymentRequest)

DescribeDeploymentResponse = _reflection.GeneratedProtocolMessageType('DescribeDeploymentResponse', (_message.Message,), {
  'DESCRIPTOR' : _DESCRIBEDEPLOYMENTRESPONSE,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DescribeDeploymentResponse)
  })
_sym_db.RegisterMessage(DescribeDeploymentResponse)

ListDeploymentsRequest = _reflection.GeneratedProtocolMessageType('ListDeploymentsRequest', (_message.Message,), {

  'LabelsEntry' : _reflection.GeneratedProtocolMessageType('LabelsEntry', (_message.Message,), {
    'DESCRIPTOR' : _LISTDEPLOYMENTSREQUEST_LABELSENTRY,
    '__module__' : 'deployment_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.ListDeploymentsRequest.LabelsEntry)
    })
  ,
  'DESCRIPTOR' : _LISTDEPLOYMENTSREQUEST,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.ListDeploymentsRequest)
  })
_sym_db.RegisterMessage(ListDeploymentsRequest)
_sym_db.RegisterMessage(ListDeploymentsRequest.LabelsEntry)

ListDeploymentsResponse = _reflection.GeneratedProtocolMessageType('ListDeploymentsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTDEPLOYMENTSRESPONSE,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.ListDeploymentsResponse)
  })
_sym_db.RegisterMessage(ListDeploymentsResponse)


_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG_CONFIGENTRY._options = None
_DEPLOYMENT_ANNOTATIONSENTRY._options = None
_DEPLOYMENT_LABELSENTRY._options = None
_LISTDEPLOYMENTSREQUEST_LABELSENTRY._options = None
# @@protoc_insertion_point(module_scope)



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import logging

from ruamel.yaml import YAML

from bentoml.deployment.utils import (
    exception_to_return_status,
    ensure_deploy_api_name_exists_in_bento,
)
from bentoml.proto.repository_pb2 import GetBentoRequest, BentoUri
from bentoml.utils import Path
from bentoml.deployment.operator import DeploymentOperatorBase
from bentoml.utils.tempdir import TempDirectory
from bentoml.yatai.status import Status
from bentoml.exceptions import BentoMLException
from bentoml.proto.deployment_pb2 import (
    ApplyDeploymentResponse,
    DescribeDeploymentResponse,
    DeleteDeploymentResponse,
    DeploymentState,
    Deployment,
)
from bentoml.deployment.serverless.serverless_utils import (
    call_serverless_command,
    parse_serverless_info_response_to_json_string,
    init_serverless_project_dir,
)

logger = logging.getLogger(__name__)

GOOGLE_MAIN_PY_TEMPLATE_HEADER = """\
from {class_name} import load

bento_service = load()

"""

GOOGLE_FUNCTION_TEMPLATE = """\
def {api_name}(request):
    api = bento_service.get_service_api('{api_name}')

    return api.handle_request(request)

"""


def generate_gcp_handler_functions_config(apis):
    function_list = {}
    for api in apis:
        function_list[api.name] = {"handler": api.name, "events": [{"http": "path"}]}
    return function_list


def generate_gcp_function_serverless_config(
    deployment_name, api_names, serverless_project_dir, region, stage
):
    config_path = os.path.join(serverless_project_dir, "serverless.yml")
    if os.path.isfile(config_path):
        os.remove(config_path)
    yaml = YAML()
    serverless_config = {
        "service": deployment_name,
        "provider": {
            "region": region,
            "stage": stage,
            "name": 'google',
            "project": deployment_name,
        },
        "functions": {
            api_name: {"handler": api_name, "events": [{"http": "path"}]}
            for api_name in api_names
        },
    }

    yaml.dump(serverless_config, Path(config_path))


def generate_gcp_function_main_py(bento_name, api_names, output_path):
    with open(os.path.join(output_path, "main.py"), "w") as f:
        f.write(GOOGLE_MAIN_PY_TEMPLATE_HEADER.format(class_name=bento_name))
        for api_name in api_names:
            api_content = GOOGLE_FUNCTION_TEMPLATE.format(api_name=api_name)
            f.write(api_content)


class GcpFunctionDeploymentOperator(DeploymentOperatorBase):
    def apply(self, deployment_pb, yatai_service, prev_deployment=None):
        try:
            deployment_spec = deployment_pb.spec
            gcp_config = deployment_spec.gcp_function_operator_config
            bento_pb = yatai_service.GetBento(
                GetBentoRequest(
                    bento_name=deployment_spec.bento_name,
                    bento_version=deployment_spec.bento_version,
                )
            )
            if bento_pb.bento.uri.type != BentoUri.LOCAL:
                raise BentoMLException(
                    'BentoML currently only support local repository'
                )
            else:
                bento_path = bento_pb.bento.uri.uri
            bento_service_metadata = bento_pb.bento.bento_service_metadata

            api_names = (
                [gcp_config.api_name]
                if gcp_config.api_name
                else [api.name for api in bento_service_metadata.apis]
            )
            ensure_deploy_api_name_exists_in_bento(
                [api.name for api in bento_service_metadata.apis], api_names
            )
            with TempDirectory() as serverless_project_dir:
                init_serverless_project_dir(
                    serverless_project_dir,
                    bento_path,
                    deployment_pb.name,
                    deployment_spec.bento_name,
                    'google-python',
                )
                generate_gcp_function_main_py(
                    deployment_spec.bento_name, api_names, serverless_project_dir
                )
                generate_gcp_function_serverless_config(
                    deployment_pb.name,
                    api_names,
                    serverless_project_dir,
                    gcp_config.region,
                    # BentoML namespace is mapping to serverless stage.
                    stage=deployment_pb.namespace,
                )
                call_serverless_command(["deploy"], serverless_project_dir)

            res_deployment_pb = Deployment(state=DeploymentState())
            res_deployment_pb.CopyFrom(deployment_pb)
            state = self.describe(res_deployment_pb, yatai_service).state
            res_deployment_pb.state.CopyFrom(state)

            return ApplyDeploymentResponse(
                status=Status.OK(), deployment=res_deployment_pb
            )
        except BentoMLException as error:
            return ApplyDeploymentResponse(status=exception_to_return_status(error))

    def describe(self, deployment_pb, yatai_service=None):
        try:
            deployment_spec = deployment_pb.spec
            gcp_config = deployment_spec.gcp_function_operator_config

            bento_pb = yatai_service.GetBento(
                GetBentoRequest(
                    bento_name=deployment_spec.bento_name,
                    bento_version=deployment_spec.bento_version,
                )
            )
            bento_service_metadata = bento_pb.bento.bento_service_metadata
            api_names = (
                [gcp_config.api_name]
                if gcp_config.api_name
                else [api.name for api in bento_service_metadata.apis]
            )
            with TempDirectory() as serverless_project_dir:
                generate_gcp_function_serverless_config(
                    deployment_pb.name,
                    api_names,
                    serverless_project_dir,
                    gcp_config.region,
                    # BentoML namespace is mapping to serverless stage.
                    stage=deployment_pb.namespace,
                )
                try:
                    response = call_serverless_command(["info"], serverless_project_dir)
                    info_json = parse_serverless_info_response_to_json_string(response)
                    state = DeploymentState(
                        state=DeploymentState.RUNNING, info_json=info_json
                    )
                    state.timestamp.GetCurrentTime()
                except BentoMLException as e:
                    state = DeploymentState(
                        state=DeploymentState.ERROR, error_message=str(e)
                    )
                    state.timestamp.GetCurrentTime()

            return DescribeDeploymentResponse(status=Status.OK(), state=state)
        except BentoMLException as error:
            return DescribeDeploymentResponse(status=exception_to_return_status(error))

    def delete(self, deployment_pb, yatai_service=None):
        try:
            state = self.describe(deployment_pb, yatai_service).state
            if state.state != DeploymentState.RUNNING:
                message = (
                    'Failed to delete, no active deployment {name}. '
                    'The current state is {state}'.format(
                        name=deployment_pb.name,
                        state=DeploymentState.State.Name(state.state),
                    )
                )
                return DeleteDeploymentResponse(status=Status.ABORTED(message))

            deployment_spec = deployment_pb.spec
            gcp_config = deployment_spec.gcp_function_operator_config

            bento_pb = yatai_service.GetBento(
                GetBentoRequest(
                    bento_name=deployment_spec.bento_name,
                    bento_version=deployment_spec.bento_version,
                )
            )
            bento_service_metadata = bento_pb.bento.bento_service_metadata
            api_names = (
                [gcp_config.api_name]
                if gcp_config.api_name
                else [api.name for api in bento_service_metadata.apis]
            )
            with TempDirectory() as serverless_project_dir:
                generate_gcp_function_serverless_config(
                    deployment_pb.name,
                    api_names,
                    serverless_project_dir,
                    gcp_config.region,
                    # BentoML namespace is mapping to serverless stage.
                    stage=deployment_pb.namespace,
                )
                try:
                    response = call_serverless_command(
                        ['remove'], serverless_project_dir
                    )
                    if "Serverless: Stack removal finished..." in response:
                        status = Status.OK()
                    else:
                        status = Status.ABORTED()
                except BentoMLException as e:
                    status = Status.INTERNAL(str(e))

            return DeleteDeploymentResponse(status=status)
        except BentoMLException as error:
            return DeleteDeploymentResponse(status=exception_to_return_status(error))



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from bentoml.handlers.base_handlers import BentoHandler
from bentoml.handlers.dataframe_handler import DataframeHandler
from bentoml.handlers.pytorch_tensor_handler import PytorchTensorHandler
from bentoml.handlers.tensorflow_tensor_handler import TensorflowTensorHandler
from bentoml.handlers.json_handler import JsonHandler
from bentoml.handlers.image_handler import ImageHandler
from bentoml.handlers.fastai_image_handler import FastaiImageHandler

__all__ = [
    "BentoHandler",
    "DataframeHandler",
    "PytorchTensorHandler",
    "TensorflowTensorHandler",
    "JsonHandler",
    "ImageHandler",
    "FastaiImageHandler",
]



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os

ARTIFACTS_SUBPATH = "artifacts"


class BentoServiceArtifact(object):
    """
    Artifact is a spec describing how to pack and load different types
    of model dependencies to/from file system.

    A call to pack and load should return an BentoServiceArtifactWrapper that can
    be saved to file system or retrieved for BentoService workload
    """

    def __init__(self, name):
        self._name = name

    @property
    def name(self):
        """
        The name of an artifact is used to set parameters for #pack when used
        in BentoService, and as name of sub-directory for the artifact files
        when saving to file system
        """
        return self._name

    def pack(self, data):
        """
        Pack the in-memory representation of the target artifacts and make sure
        they are ready for 'save' and 'get'

        Note: add "# pylint:disable=arguments-differ" to child class's pack method
        """

    def load(self, path):
        """
        Load artifact assuming it was 'self.save' on the given base_path
        """


class BentoServiceArtifactWrapper(object):
    """
    BentoServiceArtifactWrapper is an object representing a materialized Artifact,
    either loaded from file system or packed with data in a python session
    """

    def __init__(self, spec):
        self._spec = spec

    @property
    def spec(self):
        """
        :return: reference to the BentoServiceArtifact that generated this
        BentoServiceArtifactWrapper
        """
        return self._spec

    def save(self, dst):
        """
        Save artifact to given dst path
        """

    def get(self):
        """
        Get returns a reference to the artifact being packed
        """


class ArtifactCollection(dict):
    """
    A dict of artifact instances (artifact.name -> artifact_instance)
    """

    def __setitem__(self, key, artifact):
        if key != artifact.spec.name:
            raise ValueError(
                "Must use Artifact name as key, {} not equal to {}".format(
                    key, artifact.spec.name
                )
            )

        self.add(artifact)

    def __getattr__(self, item):
        return self[item].get()

    def add(self, artifact):
        if not isinstance(artifact, BentoServiceArtifactWrapper):
            raise TypeError(
                "ArtifactCollection only accepts type BentoServiceArtifactWrapper,"
                "Must call BentoServiceArtifact#pack or BentoServiceArtifact#load "
                "before adding to an ArtifactCollection"
            )

        super(ArtifactCollection, self).__setitem__(artifact.spec.name, artifact)

    def save(self, dst):
        """
        bulk operation for saving all artifacts in self.values() to `dst` path
        """
        save_path = os.path.join(dst, ARTIFACTS_SUBPATH)
        os.mkdir(save_path)
        for artifact in self.values():
            artifact.save(save_path)

    @classmethod
    def load(cls, path, artifacts_spec):
        """bulk operation for loading all artifacts from path based on a list of
        BentoServiceArtifact
        """
        load_path = os.path.join(path, ARTIFACTS_SUBPATH)
        artifacts = cls()

        for artifact_spec in artifacts_spec:
            artifact_instance = artifact_spec.load(load_path)
            artifacts.add(artifact_instance)

        return artifacts



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from cerberus import Validator

from bentoml.utils import ProtoMessageToDict
from bentoml.proto.deployment_pb2 import DeploymentSpec

deployment_schema = {
    'name': {'type': 'string', 'required': True, 'minlength': 4},
    'namespace': {'type': 'string', 'required': True, 'minlength': 4},
    'labels': {'type': 'dict', 'allow_unknown': True},
    'annotations': {'type': 'dict', 'allow_unknown': True},
    'spec': {
        'type': 'dict',
        'required': True,
        'schema': {
            'operator': {
                'type': 'string',
                'required': True,
                'allowed': DeploymentSpec.DeploymentOperator.keys(),
            },
            'bento_name': {'type': 'string', 'required': True},
            'bento_version': {'type': 'string', 'required': True},
            'custom_operator_config': {
                'type': 'dict',
                'schema': {
                    'name': {'type': 'string'},
                    'config': {'type': 'dict', 'allow_unknown': True},
                },
            },
            'sagemaker_operator_config': {
                'type': 'dict',
                'schema': {
                    'region': {'type': 'string'},
                    'api_name': {'type': 'string', 'required': True},
                    'instance_type': {'type': 'string'},
                    'instance_count': {'type': 'integer', 'min': 1},
                },
            },
            'aws_lambda_operator_config': {
                'type': 'dict',
                'schema': {
                    'region': {'type': 'string', 'required': True},
                    'api_name': {'type': 'string'},
                },
            },
            'gcp_function_operator_config': {
                'type': 'dict',
                'schema': {
                    'region': {'type': 'string', 'required': True},
                    'api_name': {'type': 'string'},
                },
            },
            'kubernetes_operator_config': {
                'type': 'dict',
                'schema': {
                    'kube_namespace': {'type': 'string'},
                    'replicas': {'type': 'integer', 'min': 1},
                    'service_name': {'type': 'string'},
                    'service_type': {'type': 'string'},
                },
            },
        },
    },
    'state': {
        'type': 'dict',
        'schema': {
            'state': {'type': 'integer'},
            'error_message': {'type': 'string'},
            'info_json': {'type': 'string'},
        },
    },
}


def validate_pb_schema(pb, schema):
    pb_dict = ProtoMessageToDict(pb)
    v = Validator(schema)
    if v.validate(pb_dict):
        return None
    else:
        return v.errors


def validate_deployment_pb_schema(pb):
    return validate_pb_schema(pb, deployment_schema)



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from flask import Flask, Response, request
from bentoml.utils.usage_stats import track_server


def setup_bento_service_api_route(app, api):
    def view_function():
        return api.handle_request(request)

    app.add_url_rule(
        rule="/invocations",
        endpoint=api.name,
        view_func=view_function,
        methods=["POST"],
    )


def ping_view_func():
    return Response(response="\n", status=200, mimetype="application/json")


def setup_routes(app, bento_service, api_name):
    """
    Setup routes required for AWS sagemaker
    /ping
    /invocations
    """
    app.add_url_rule("/ping", "ping", ping_view_func)
    api = bento_service.get_service_api(api_name)
    setup_bento_service_api_route(app, api)


class BentoSagemakerServer:
    """
    BentoSagemakerServer create an AWS Sagemaker compatibility rest API server.
    """

    _DEFAULT_PORT = 8080

    def __init__(self, bento_service, api_name, app_name=None):
        app_name = bento_service.name if app_name is None else app_name

        self.bento_service = bento_service
        self.app = Flask(app_name)
        setup_routes(self.app, self.bento_service, api_name)

    def start(self):
        track_server('sagemaker')
        self.app.run(port=BentoSagemakerServer._DEFAULT_PORT)



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import re
import os
import importlib

from six.moves.urllib.parse import urlparse, uses_netloc, uses_params, uses_relative
from google.protobuf.json_format import MessageToJson, MessageToDict
from ruamel.yaml import YAML

from bentoml import __version__ as BENTOML_VERSION, _version as version_mod

try:
    from pathlib import Path

    Path().expanduser()
except (ImportError, AttributeError):
    from pathlib2 import Path

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

_VALID_URLS = set(uses_relative + uses_netloc + uses_params)
_VALID_URLS.discard("")


def is_url(url):
    try:
        return urlparse(url).scheme in _VALID_URLS
    except Exception:  # pylint:disable=broad-except
        return False


def isidentifier(s):
    """
    Return true if string is in a valid python identifier format:

    https://docs.python.org/2/reference/lexical_analysis.html#identifiers
    """
    try:
        return s.isidentifier()
    except AttributeError:
        # str#isidentifier is only available in python 3
        return re.match(r"[A-Za-z_][A-Za-z_0-9]*\Z", s) is not None


def dump_to_yaml_str(yaml_dict):
    yaml = YAML()
    string_io = StringIO()
    yaml.dump(yaml_dict, string_io)
    return string_io.getvalue()


def pb_to_yaml(message):
    message_dict = MessageToJson(message)
    return dump_to_yaml_str(message_dict)


def ProtoMessageToDict(protobuf_msg, **kwargs):
    if 'preserving_proto_field_name' not in kwargs:
        kwargs['preserving_proto_field_name'] = True

    return MessageToDict(protobuf_msg, **kwargs)


def _is_pypi_release():
    is_installed_package = hasattr(version_mod, 'version_json')
    is_tagged = not BENTOML_VERSION.startswith('0+untagged')
    is_clean = not version_mod.get_versions()['dirty']
    return is_installed_package and is_tagged and is_clean


# this is for BentoML developer to create BentoService containing custom develop
# branches of BentoML library, it gets triggered when BentoML module is installed
# via "pip install --editable ."
def _is_bentoml_in_develop_mode():
    try:
        module_location, = importlib.util.find_spec(
            'bentoml'
        ).submodule_search_locations
    except AttributeError:
        # python 2.7 doesn't have importlib.util, will fall back to imp instead
        import imp

        _, module_location, _ = imp.find_module('bentoml')

    setup_py_path = os.path.abspath(os.path.join(module_location, '..', 'setup.py'))
    return not _is_pypi_release() and os.path.isfile(setup_py_path)



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

DEFAULT_NGINX_CONFIG = """\
worker_processes 1;
daemon off; # Prevent forking

pid /tmp/nginx.pid;
error_log /var/log/nginx/error.log;

events {
  # defaults
}

http {
  include /etc/nginx/mime.types;
  default_type application/octet-stream;
  access_log /var/log/nginx/access.log combined;

  upstream gunicorn {
    server unix:/tmp/gunicorn.sock;
  }

  server {
    listen 8080 deferred;
    client_max_body_size 500m;

    keepalive_timeout 5;
    proxy_read_timeout 1200s;

    location ~ ^/(ping|invocations) {
      proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
      proxy_set_header Host $http_host;
      proxy_redirect off;
      proxy_pass http://gunicorn;
    }

    location / {
      return 404 "{}";
    }
  }
}
"""

DEFAULT_WSGI_PY = """\
import os

from bentoml.archive import load
from bentoml.server.bento_sagemaker_server import BentoSagemakerServer

api_name = os.environ.get('API_NAME', None)
model_service = load('/opt/program')
server = BentoSagemakerServer(model_service, api_name)
app = server.app
"""

DEFAULT_SERVE_SCRIPT = """\
#!/usr/bin/env python

# This implement the sagemaker serving service shell.  It starts nginx and gunicorn.
# Parameter                    Env Var                          Default Value
# number of workers            BENTO_SERVER_TIMEOUT             60s
# timeout                      BENTO_SERVER_WORKERS             number of cpu cores / 2 + 1
# api name                     API_NAME                         None

import subprocess
import os
import signal
import sys

from bentoml.server.utils import get_bento_recommend_gunicorn_worker_count

bento_server_timeout = os.environ.get('BENTO_SERVER_TIMEOUT', 60)
bento_server_workers = int(os.environ.get('BENTO_SERVER_WORKERS', get_bento_recommend_gunicorn_worker_count()))

def sigterm_handler(nginx_pid, gunicorn_pid):
    try:
        os.kill(nginx_pid, signal.SIGQUIT)
    except OSError:
        pass
    try:
        os.kill(gunicorn_pid, signal.SIGTERM)
    except OSError:
        pass

    sys.exit(0)


def _serve():
    # link the log streams to stdout/err so they will be logged to the container logs
    subprocess.check_call(['ln', '-sf', '/dev/stdout', '/var/log/nginx/access.log'])
    subprocess.check_call(['ln', '-sf', '/dev/stderr', '/var/log/nginx/error.log'])

    nginx = subprocess.Popen(['nginx', '-c', '/opt/program/nginx.conf'])
    gunicorn_app = subprocess.Popen(['gunicorn',
                                     '--timeout', str(bento_server_timeout),
                                     '-k', 'gevent',
                                     '-b', 'unix:/tmp/gunicorn.sock',
                                     '-w', str(bento_server_workers),
                                     'wsgi:app'])
    signal.signal(signal.SIGTERM, lambda a, b: sigterm_handler(nginx.pid, gunicorn_app.pid))

    pids = set([nginx.pid, gunicorn_app.pid])
    while True:
        pid, _ = os.wait()
        if pid in pids:
            break
    sigterm_handler(nginx.pid, gunicorn_app.pid)
    print('Inference server exiting')


if __name__ == '__main__':
  _serve()
"""  # noqa: E501



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import setuptools
import versioneer

with open("README.md", "r") as f:
    long_description = f.read()

install_requires = [
    "ruamel.yaml>=0.15.0",
    "numpy",
    "flask",
    "gunicorn",
    "six",
    "click>=7.0",
    "pandas",
    "dill",
    "prometheus_client",
    "python-json-logger",
    "boto3",
    "pathlib2",
    "requests",
    "packaging",
    "docker",
    "configparser",
    "sqlalchemy>=1.3.0",
    "protobuf>=3.6.0",
    "grpcio",
    "cerberus",
    "tabulate",
    "humanfriendly",
    "alembic",
]

imageio = ["imageio>=2.5.0"]
cv2 = ["opencv-python"]
pytorch = ["torch", "torchvision"]
fastai = ["fastai", "matplotlib"]
tensorflow = ["tensorflow"]
xgboost = ["xgboost"]
h2o = ["h2o"]
api_server = ["gunicorn", "prometheus_client"]

optional_requires = api_server + imageio + pytorch + tensorflow + fastai + xgboost + h2o

tests_require = (
    [
        "pytest>=4.1.0",
        "pytest-cov>=2.7.1",
        "snapshottest>=0.5.0",
        "mock>=2.0.0",
        "tox>=3.12.1",
        "coverage>=4.4",
        "codecov",
    ]
    + imageio
    + cv2
    + fastai
)

dev_requires = [
    "pylint>=2.3.1",
    "flake8",
    "tox-conda>=0.2.0",
    "twine",
    "black",
    "setuptools",
    "gitpython>=2.0.2",
    "grpcio-tools",
] + tests_require

docs_requires = [
    "sphinx",
    "sphinx-click",
    "sphinx_rtd_theme",
    "sphinxcontrib-fulltoc",
]

dev_all = install_requires + dev_requires + optional_requires + docs_requires

extras_require = {
    "all": dev_all,
    "dev": dev_requires,
    "api_server": api_server,
    "test": tests_require,

    "doc_builder": docs_requires + install_requires,  # required by readthedocs.io
}

setuptools.setup(
    name="BentoML",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    author="atalaya.io",
    author_email="contact@atalaya.io",
    description="A python framework for serving and operating machine learning models",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=install_requires,
    extras_require=extras_require,
    url="https://github.com/bentoml/BentoML",
    packages=setuptools.find_packages(exclude=["tests*"]),
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
    entry_points={"console_scripts": ["bentoml=bentoml.cli:cli"]},
    project_urls={
        "Bug Reports": "https://github.com/bentoml/BentoML/issues",
        "Source Code": "https://github.com/bentoml/BentoML",
        "Slack User Group": "https://bit.ly/2N5IpbB",
    },
    include_package_data=True,  # Required for '.cfg' files under bentoml/config
)



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function


import sys
import platform
import json
import logging
import time
import atexit

import uuid
import requests

from bentoml.utils import _is_pypi_release
from bentoml import config
from bentoml import __version__ as BENTOML_VERSION


logger = logging.getLogger(__name__)

AMPLITUDE_URL = "https://api.amplitude.com/httpapi"
PLATFORM = platform.platform()
PY_VERSION = "{major}.{minor}.{micro}".format(
    major=sys.version_info.major,
    minor=sys.version_info.minor,
    micro=sys.version_info.micro,
)
SESSION_ID = str(uuid.uuid4())  # uuid that marks current python session


# Use dev amplitude key
API_KEY = '7f65f2446427226eb86f6adfacbbf47a'
if _is_pypi_release():
    # Use prod amplitude key
    API_KEY = '1ad6ee0e81b9666761aebd55955bbd3a'


def _send_amplitude_event(event_type, event_properties):
    """Send event to amplitude
    https://developers.amplitude.com/?java#keys-for-the-event-argument
    """
    event = [
        {
            "event_type": event_type,
            "user_id": SESSION_ID,
            "event_properties": event_properties,
        }
    ]
    event_data = {"api_key": API_KEY, "event": json.dumps(event)}

    try:
        return requests.post(AMPLITUDE_URL, data=event_data, timeout=1)
    except Exception as err:  # pylint:disable=broad-except
        # silently fail since this error does not concern BentoML end users
        logger.debug(str(err))


def _get_bento_service_event_properties(bento_service, properties=None):
    if properties is None:
        properties = {}

    artifact_types = []
    handler_types = []

    for artifact in bento_service.artifacts.items():
        artifact_instance = artifact[1]
        artifact_types.append(artifact_instance.__class__.__name__)

    for api in bento_service.get_service_apis():
        handler_types.append(api.handler.__class__.__name__)

    properties["handler_types"] = handler_types
    properties["artifact_types"] = artifact_types
    properties["env"] = bento_service.env.to_dict()
    return properties


def track(event_type, event_properties=None):
    if not config().getboolean("core", "usage_tracking"):
        return  # Usage tracking disabled

    if event_properties is None:
        event_properties = {}

    event_properties['py_version'] = PY_VERSION
    event_properties["bento_version"] = BENTOML_VERSION
    event_properties["platform_info"] = PLATFORM

    return _send_amplitude_event(event_type, event_properties)


def track_save(bento_service, extra_properties=None):
    properties = _get_bento_service_event_properties(bento_service, extra_properties)
    return track("save", properties)


def track_load_start():
    return track('load-start', {})


def track_load_finish(bento_service):
    properties = _get_bento_service_event_properties(bento_service)
    return track("load", properties)


def track_cli(command, deploy_platform=None):
    properties = {}
    if deploy_platform is not None:
        properties['platform'] = deploy_platform
    return track('cli-' + command, properties)


def track_server(server_type, extra_properties=None):
    properties = extra_properties or {}

    # track server start event
    track('server-{server_type}-start'.format(server_type=server_type), properties)

    start_time = time.time()

    @atexit.register
    def log_exit():  # pylint: disable=unused-variable
        # track server stop event
        duration = time.time() - start_time
        properties['uptime'] = int(duration)
        return track(
            'server-{server_type}-stop'.format(server_type=server_type), properties
        )



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import sys
import logging


from bentoml.utils import dump_to_yaml_str
from bentoml.utils.usage_stats import track_load_finish, track_load_start
from bentoml.exceptions import BentoMLException
from bentoml.archive.config import BentoArchiveConfig
from bentoml.proto.repository_pb2 import BentoServiceMetadata

LOG = logging.getLogger(__name__)


def load_bentoml_config(archive_path):
    try:
        return BentoArchiveConfig.load(os.path.join(archive_path, "bentoml.yml"))
    except FileNotFoundError:
        raise ValueError(
            "BentoML can't locate config file 'bentoml.yml'"
            " in archive path: {}".format(archive_path)
        )


def load_bento_service_metadata(archive_path):
    config = load_bentoml_config(archive_path)

    bento_service_metadata = BentoServiceMetadata()
    bento_service_metadata.name = config["metadata"]["service_name"]
    bento_service_metadata.version = config["metadata"]["service_version"]
    bento_service_metadata.created_at.FromDatetime(config["metadata"]["created_at"])

    if "env" in config:
        if "setup_sh" in config["env"]:
            bento_service_metadata.env.setup_sh = config["env"]["setup_sh"]

        if "conda_env" in config["env"]:
            bento_service_metadata.env.conda_env = dump_to_yaml_str(
                config["env"]["conda_env"]
            )

        if "pip_dependencies" in config["env"]:
            bento_service_metadata.env.pip_dependencies = "\n".join(
                config["env"]["pip_dependencies"]
            )
        if "python_version" in config["env"]:
            bento_service_metadata.env.python_version = config["env"]["python_version"]

    if "apis" in config:
        for api_config in config["apis"]:
            api_metadata = BentoServiceMetadata.BentoServiceApi()
            if "name" in api_config:
                api_metadata.name = api_config["name"]
            if "handler_type" in api_config:
                api_metadata.handler_type = api_config["handler_type"]
            if "docs" in api_config:
                api_metadata.docs = api_config["docs"]
            bento_service_metadata.apis.extend([api_metadata])

    if "artifacts" in config:
        for artifact_config in config["artifacts"]:
            artifact_metadata = BentoServiceMetadata.BentoArtifact()
            if "name" in artifact_config:
                artifact_metadata.name = artifact_config["name"]
            if "artifact_type" in artifact_config:
                artifact_metadata.artifact_type = artifact_config["artifact_type"]
            bento_service_metadata.artifacts.extend([artifact_metadata])

    return bento_service_metadata


def load_bento_service_class(archive_path):
    """
    Load a BentoService class from saved archive in given path

    :param archive_path: A path to Bento files generated from BentoService#save,
        #save_to_dir, or the path to pip installed BentoArchive directory
    :return: BentoService class
    """
    config = load_bentoml_config(archive_path)
    metadata = config["metadata"]

    # Load target module containing BentoService class from given path
    module_file_path = os.path.join(
        archive_path, metadata["service_name"], metadata["module_file"]
    )
    if not os.path.isfile(module_file_path):
        # Try loading without service_name prefix, for loading from a installed PyPi
        module_file_path = os.path.join(archive_path, metadata["module_file"])

    if not os.path.isfile(module_file_path):
        raise BentoMLException(
            "Can not locate module_file {} in archive {}".format(
                metadata["module_file"], archive_path
            )
        )

    # Prepend archive_path to sys.path for loading extra python dependencies
    sys.path.insert(0, archive_path)

    module_name = metadata["module_name"]
    if module_name in sys.modules:
        LOG.warning(
            "Module `%s` already loaded, using existing imported module.", module_name
        )
        module = sys.modules[module_name]
    elif sys.version_info >= (3, 5):
        import importlib.util

        spec = importlib.util.spec_from_file_location(module_name, module_file_path)
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)
    elif sys.version_info >= (3, 3):
        from importlib.machinery import SourceFileLoader

        # pylint:disable=deprecated-method
        module = SourceFileLoader(module_name, module_file_path).load_module(
            module_name
        )
        # pylint:enable=deprecated-method
    else:
        import imp

        module = imp.load_source(module_name, module_file_path)

    # Remove archive_path from sys.path to avoid import naming conflicts
    sys.path.remove(archive_path)

    model_service_class = module.__getattribute__(metadata["service_name"])
    # Set _bento_archive_path, which tells BentoService where to load its artifacts
    model_service_class._bento_archive_path = archive_path
    # Set cls._version, service instance can access it via svc.version
    model_service_class._bento_service_version = metadata["service_version"]

    return model_service_class


def load(archive_path):
    """Load bento service from local file path or s3 path

    Args:
        archive_path (str): The path that contains archived bento service.
            It could be local file path or aws s3 path

    Returns:
        bentoml.service.BentoService: The loaded bento service.
    """
    track_load_start()
    svc_cls = load_bento_service_class(archive_path)

    svc = svc_cls.load_from_dir(archive_path)
    track_load_finish(svc)
    return svc


def load_service_api(archive_path, api_name=None):
    bento_service = load(archive_path)
    return bento_service.get_service_api(api_name)



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from bentoml.proto.status_pb2 import Status as StatusProto


class Status(object):
    @staticmethod
    def OK(message=None):
        return StatusProto(status_code=StatusProto.OK, error_message=message)

    @staticmethod
    def CANCELLED(message=None):
        return StatusProto(status_code=StatusProto.CANCELLED, error_message=message)

    @staticmethod
    def UNKNOWN(message=None):
        return StatusProto(status_code=StatusProto.UNKNOWN, error_message=message)

    @staticmethod
    def INVALID_ARGUMENT(message=None):
        return StatusProto(
            status_code=StatusProto.INVALID_ARGUMENT, error_message=message
        )

    @staticmethod
    def DEADLINE_EXCEEDED(message=None):
        return StatusProto(
            status_code=StatusProto.DEADLINE_EXCEEDED, error_message=message
        )

    @staticmethod
    def NOT_FOUND(message=None):
        return StatusProto(status_code=StatusProto.NOT_FOUND, error_message=message)

    @staticmethod
    def ALREADY_EXISTS(message=None):
        return StatusProto(
            status_code=StatusProto.ALREADY_EXISTS, error_message=message
        )

    @staticmethod
    def PERMISSION_DENIED(message=None):
        return StatusProto(
            status_code=StatusProto.PERMISSION_DENIED, error_message=message
        )

    @staticmethod
    def UNAUTHENTICATED(message=None):
        return StatusProto(
            status_code=StatusProto.UNAUTHENTICATED, error_message=message
        )

    @staticmethod
    def RESOURCE_EXHAUSTED(message=None):
        return StatusProto(
            status_code=StatusProto.RESOURCE_EXHAUSTED, error_message=message
        )

    @staticmethod
    def FAILED_RECONDITION(message=None):
        return StatusProto(
            status_code=StatusProto.FAILED_PRECONDITION, error_message=message
        )

    @staticmethod
    def ABORTED(message=None):
        return StatusProto(status_code=StatusProto.ABORTED, error_message=message)

    @staticmethod
    def OUT_OF_RANGE(message=None):
        return StatusProto(status_code=StatusProto.OUT_OF_RANGE, error_message=message)

    @staticmethod
    def UNIMPLEMENTED(message=None):
        return StatusProto(status_code=StatusProto.UNIMPLEMENTED, error_message=message)

    @staticmethod
    def INTERNAL(message=None):
        return StatusProto(status_code=StatusProto.INTERNAL, error_message=message)

    @staticmethod
    def UNAVAILABLE(message=None):
        return StatusProto(status_code=StatusProto.UNAVAILABLE, error_message=message)

    @staticmethod
    def DATA_LOSS(message=None):
        return StatusProto(status_code=StatusProto.DATA_LOSS, error_message=message)



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import json
import os
import logging
import importlib
import shutil
import subprocess

from setuptools import sandbox

from bentoml.exceptions import (
    BentoMLException,
    BentoMLMissingDepdencyException,
    BentoMLInvalidArgumentException,
)

logger = logging.getLogger(__name__)


def process_docker_api_line(payload):
    """ Process the output from API stream, throw an Exception if there is an error """
    # Sometimes Docker sends to "{}\n" blocks together...
    for segment in payload.decode("utf-8").split("\n"):
        line = segment.strip()
        if line:
            try:
                line_payload = json.loads(line)
            except ValueError as e:
                print("Could not decipher payload from Docker API: " + e)
            if line_payload:
                if "errorDetail" in line_payload:
                    error = line_payload["errorDetail"]
                    logger.error(error['message'])
                    raise RuntimeError(
                        "Error on build - code: {code}, message: {message}".format(
                            code=error["code"], message=error['message']
                        )
                    )
                elif "stream" in line_payload:
                    logger.info(line_payload['stream'])


def _find_bentoml_module_location():
    try:
        module_location, = importlib.util.find_spec(
            'bentoml'
        ).submodule_search_locations
    except AttributeError:
        # python 2.7 doesn't have importlib.util, will fall back to imp instead
        import imp

        _, module_location, _ = imp.find_module('bentoml')
    return module_location


def add_local_bentoml_package_to_repo(archive_path):
    module_location = _find_bentoml_module_location()
    bentoml_setup_py = os.path.abspath(os.path.join(module_location, '..', 'setup.py'))

    assert os.path.isfile(bentoml_setup_py), '"setup.py" for Bentoml module not found'

    # Create tmp directory inside bentoml module for storing the bundled
    # targz file. Since dist-dir can only be inside of the module directory
    bundle_dir_name = '__bento_dev_tmp'
    source_dir = os.path.abspath(os.path.join(module_location, '..', bundle_dir_name))
    if os.path.isdir(source_dir):
        shutil.rmtree(source_dir, ignore_errors=True)
    os.mkdir(source_dir)

    sandbox.run_setup(
        bentoml_setup_py, ['sdist', '--format', 'gztar', '--dist-dir', bundle_dir_name]
    )

    # copy the generated targz to archive directory and remove it from
    # bentoml module directory
    dest_dir = os.path.join(archive_path, 'bundled_pip_dependencies')
    shutil.copytree(source_dir, dest_dir)
    shutil.rmtree(source_dir)


def ensure_docker_available_or_raise():
    try:
        subprocess.check_output(['docker', 'info'])
    except subprocess.CalledProcessError as error:
        raise BentoMLException(
            'Error executing docker command: {}'.format(error.output)
        )
    except FileNotFoundError:
        raise BentoMLMissingDepdencyException(
            'Docker is required for this deployment. Please visit '
            'www.docker.come for instructions'
        )


def ensure_deploy_api_name_exists_in_bento(all_api_names, deployed_api_names):
    if not set(deployed_api_names).issubset(all_api_names):
        raise BentoMLInvalidArgumentException(
            "Expect api names {deployed_api_names} to be "
            "subset of {all_api_names}".format(
                deployed_api_names=deployed_api_names, all_api_names=all_api_names
            )
        )


def exception_to_return_status(error):
    from bentoml.yatai.status import Status

    if type(error) is BentoMLInvalidArgumentException:
        return Status.INVALID_ARGUMENT(str(error))
    else:
        return Status.INTERNAL(str(error))



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
from sys import version_info
from ruamel.yaml import YAML

from bentoml.utils import Path, StringIO
from bentoml.configuration import get_bentoml_deploy_version

PYTHON_VERSION = "{major}.{minor}.{micro}".format(
    major=version_info.major, minor=version_info.minor, micro=version_info.micro
)

CONDA_ENV_BASE_YAML = """
name: {name}
channels:
  - defaults
dependencies:
  - python={python_version}
  - pip
  - pip:
    - bentoml[api_server]=={bentoml_version}
"""

CONDA_ENV_DEFAULT_NAME = "bentoml-custom-conda-env"


class CondaEnv(object):
    """A wrapper around conda environment settings file, allows adding/removing
    conda or pip dependencies to env configuration, and supports load/export those
    settings from/to yaml files. The generated file is the same format as yaml file
    generated from `conda env export` command.
    """

    def __init__(
        self,
        name=CONDA_ENV_DEFAULT_NAME,
        python_version=PYTHON_VERSION,
        bentoml_version=None,
    ):
        self._yaml = YAML()
        self._yaml.default_flow_style = False

        if bentoml_version is None:
            bentoml_version = get_bentoml_deploy_version()

        self._conda_env = self._yaml.load(
            CONDA_ENV_BASE_YAML.format(
                name=name,
                python_version=python_version,
                bentoml_version=bentoml_version,
            )
        )

    def set_name(self, name):
        self._conda_env["name"] = name

    def get_name(self):
        return self._conda_env["name"]

    def add_conda_dependencies(self, extra_conda_dependencies):
        self._conda_env["dependencies"] += extra_conda_dependencies

    def add_pip_dependencies(self, extra_pip_dependencies):
        for dep in self._conda_env["dependencies"]:
            if isinstance(dep, dict) and "pip" in dep:
                # there is already a pip list in conda_env, append extra deps
                dep["pip"] += extra_pip_dependencies
                return self

        self._conda_env["dependencies"] += [{"pip": extra_pip_dependencies}]

    def add_channels(self, channels):
        self._conda_env["channels"] += channels

    def to_yaml_str(self):
        string_io = StringIO()
        self._yaml.dump(self._conda_env, string_io)
        return string_io.getvalue()

    def write_to_yaml_file(self, filepath):
        output_yaml = Path(filepath)
        self._yaml.dump(self._conda_env, output_yaml)

    @classmethod
    def from_yaml(cls, yaml_str):
        new_conda_env = cls()
        new_conda_env._conda_env = new_conda_env._yaml.load(yaml_str)
        return new_conda_env

    @classmethod
    def from_current_conda_env(cls):
        # TODO: implement me!
        pass


class BentoServiceEnv(object):
    """Defines all aspect of the system environment requirements for a custom
    BentoService to be used. This includes:

    conda environment - for most python third-party packages and libraries
    requirements_txt  - for pypi dependencies that can be resolved by pip
        when exported BentoArchieve is installed as a pypi package
    setup_sh - for customizing the environment with user defined bash script
    """

    def __init__(self):
        bentoml_deploy_version = get_bentoml_deploy_version()
        self._conda_env = CondaEnv(bentoml_version=bentoml_deploy_version)
        self._pip_dependencies = ["bentoml=={}".format(bentoml_deploy_version)]
        self._python_version = PYTHON_VERSION

        self._setup_sh = None

    def get_conda_env_name(self):
        return self._conda_env.get_name()

    def set_conda_env_name(self, name):
        self._conda_env.set_name(name)

    def add_conda_channels(self, channels):
        if not isinstance(channels, list):
            channels = [channels]
        self._conda_env.add_channels(channels)

    def add_conda_dependencies(self, conda_dependencies):
        if not isinstance(conda_dependencies, list):
            conda_dependencies = [conda_dependencies]
        self._conda_env.add_conda_dependencies(conda_dependencies)

    def add_conda_pip_dependencies(self, pip_dependencies):
        if not isinstance(pip_dependencies, list):
            pip_dependencies = [pip_dependencies]
        self._conda_env.add_pip_dependencies(pip_dependencies)

    def add_handler_dependencies(self, handler_dependencies):
        if not isinstance(handler_dependencies, list):
            handler_dependencies = [handler_dependencies]
        self._pip_dependencies += handler_dependencies

    def set_setup_sh(self, setup_sh_path_or_content):
        setup_sh_file = Path(setup_sh_path_or_content)

        if setup_sh_file.is_file():
            with setup_sh_file.open("rb") as f:
                self._setup_sh = f.read()
        else:
            self._setup_sh = setup_sh_path_or_content.encode("utf-8")

    def add_pip_dependencies(self, pip_dependencies):
        if not isinstance(pip_dependencies, list):
            pip_dependencies = [pip_dependencies]
        self._pip_dependencies += pip_dependencies

    def set_requirements_txt(self, requirements_txt_path):
        requirements_txt_file = Path(requirements_txt_path)

        with requirements_txt_file.open("rb") as f:
            content = f.read()
            module_list = content.decode("utf-8").split("\n")
            self._pip_dependencies += module_list

    def save(self, path):
        conda_yml_file = os.path.join(path, "environment.yml")
        self._conda_env.write_to_yaml_file(conda_yml_file)

        requirements_txt_file = os.path.join(path, "requirements.txt")

        with open(requirements_txt_file, "wb") as f:
            pip_content = "\n".join(self._pip_dependencies).encode("utf-8")
            f.write(pip_content)

        if self._setup_sh:
            setup_sh_file = os.path.join(path, "setup.sh")
            with open(setup_sh_file, "wb") as f:
                f.write(self._setup_sh)

    @classmethod
    def from_dict(cls, env_dict):
        env = cls()

        if "setup_sh" in env_dict:
            env.set_setup_sh(env_dict["setup_sh"])

        if "requirements_txt" in env_dict:
            env.set_requirements_txt(env_dict["requirements_txt"])

        if "pip_dependencies" in env_dict:
            env.add_pip_dependencies(env_dict["pip_dependencies"])

        if "conda_channels" in env_dict:
            env.add_conda_channels(env_dict["conda_channels"])

        if "conda_dependencies" in env_dict:
            env.add_conda_dependencies(env_dict["conda_dependencies"])

        if "conda_pip_dependencies" in env_dict:
            env.add_conda_pip_dependencies(env_dict["conda_pip_dependencies"])

        return env

    def to_dict(self):
        env_dict = dict()

        if self._setup_sh:
            env_dict["setup_sh"] = self._setup_sh

        if self._pip_dependencies:
            env_dict["pip_dependencies"] = self._pip_dependencies

        env_dict["conda_env"] = self._conda_env._conda_env

        env_dict["python_version"] = self._python_version
        return env_dict



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import sys
import shutil

from bentoml.artifact import BentoServiceArtifact, BentoServiceArtifactWrapper


def _import_fastai_module():
    try:
        import fastai
    except ImportError:
        raise ImportError(
            "fastai package is required to use " "bentoml.artifacts.FastaiModelArtifact"
        )

    return fastai


class FastaiModelArtifact(BentoServiceArtifact):
    """Saving and Loading FastAI Model

    Args:
        name (str): Name for the fastai model

    Raises:
        ImportError: Require fastai package to use Fast ai model artifact
        ValueError: Model is not instance of fast ai model
    """

    def __init__(self, name):
        if sys.version_info.major < 3 or sys.version_info.minor < 6:
            raise SystemError("fast ai requires python 3.6 version or higher")

        super(FastaiModelArtifact, self).__init__(name)
        self._file_name = name + '.pkl'

    def _model_file_path(self, base_path):
        return os.path.join(base_path, self._file_name)

    def pack(self, model):  # pylint:disable=arguments-differ
        fastai_module = _import_fastai_module()

        if not isinstance(model, fastai_module.basic_train.Learner):
            raise ValueError(
                "Expect `model` argument to be `fastai.basic_train.Learner` instance"
            )

        return _FastaiModelArtifactWrapper(self, model)

    def load(self, path):
        fastai_module = _import_fastai_module()

        model = fastai_module.basic_train.load_learner(path, self._file_name)
        return self.pack(model)


class _FastaiModelArtifactWrapper(BentoServiceArtifactWrapper):
    def __init__(self, spec, model):
        super(_FastaiModelArtifactWrapper, self).__init__(spec)

        self._model = model

    def save(self, dst):
        self._model.export(file=self.spec._file_name)

        shutil.copyfile(
            os.path.join(self._model.path, self.spec._file_name),
            self.spec._model_file_path(dst),
        )

    def get(self):
        return self._model



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import logging
import datetime

from sqlalchemy import (
    Column,
    Enum,
    String,
    Integer,
    JSON,
    Boolean,
    DateTime,
    UniqueConstraint,
)
from sqlalchemy.orm.exc import NoResultFound
from google.protobuf.json_format import ParseDict

from bentoml.utils import ProtoMessageToDict
from bentoml.exceptions import BentoMLRepositoryException
from bentoml.db import Base, create_session
from bentoml.proto.repository_pb2 import (
    UploadStatus,
    BentoUri,
    BentoServiceMetadata,
    Bento as BentoPB,
)

logger = logging.getLogger(__name__)


DEFAULT_UPLOAD_STATUS = UploadStatus(status=UploadStatus.UNINITIALIZED)


class Bento(Base):
    __tablename__ = 'bentos'
    __table_args__ = tuple(UniqueConstraint('name', 'version', name='_name_version_uc'))

    id = Column(Integer, primary_key=True)
    name = Column(String, nullable=False)
    version = Column(String, nullable=False)

    # Storage URI for this Bento
    uri = Column(String, nullable=False)
    uri_type = Column(Enum(*BentoUri.StorageType.keys()), default=BentoUri.UNSET)

    # JSON filed mapping directly to BentoServiceMetadata proto message
    bento_service_metadata = Column(JSON, nullable=False, default={})

    # Time of AddBento call, the time of Bento creation can be found in metadata field
    created_at = Column(DateTime, default=datetime.datetime.utcnow)

    # latest upload status, JSON message also includes last update timestamp
    upload_status = Column(
        JSON, nullable=False, default=ProtoMessageToDict(DEFAULT_UPLOAD_STATUS)
    )

    # mark as deleted
    deleted = Column(Boolean, default=False)


def _bento_orm_obj_to_pb(bento_obj):
    bento_service_metadata_pb = ParseDict(
        bento_obj.bento_service_metadata, BentoServiceMetadata()
    )
    bento_uri = BentoUri(
        uri=bento_obj.uri, type=BentoUri.StorageType.Value(bento_obj.uri_type)
    )
    return BentoPB(
        name=bento_obj.name,
        version=bento_obj.version,
        uri=bento_uri,
        bento_service_metadata=bento_service_metadata_pb,
    )


class BentoMetadataStore(object):
    def __init__(self, sess_maker):
        self.sess_maker = sess_maker

    def add(self, bento_name, bento_version, uri, uri_type):
        with create_session(self.sess_maker) as sess:
            bento_obj = Bento()
            bento_obj.name = bento_name
            bento_obj.version = bento_version
            bento_obj.uri = uri
            bento_obj.uri_type = BentoUri.StorageType.Name(uri_type)
            return sess.add(bento_obj)

    def get(self, bento_name, bento_version):
        with create_session(self.sess_maker) as sess:
            try:
                bento_obj = (
                    sess.query(Bento)
                    .filter_by(name=bento_name, version=bento_version)
                    .one()
                )
                if bento_obj.deleted:
                    # bento has been marked as deleted
                    return None
                return _bento_orm_obj_to_pb(bento_obj)
            except NoResultFound:
                return None

    def update_bento_service_metadata(
        self, bento_name, bento_version, bento_service_metadata_pb
    ):
        with create_session(self.sess_maker) as sess:
            try:
                bento_obj = (
                    sess.query(Bento)
                    .filter_by(name=bento_name, version=bento_version, deleted=False)
                    .one()
                )
                bento_obj.bento_service_metadata = ProtoMessageToDict(
                    bento_service_metadata_pb
                )
            except NoResultFound:
                raise BentoMLRepositoryException(
                    "Bento %s:%s is not found in repository" % bento_name, bento_version
                )

    def update_upload_status(self, bento_name, bento_version, upload_status_pb):
        with create_session(self.sess_maker) as sess:
            try:
                bento_obj = (
                    sess.query(Bento)
                    .filter_by(name=bento_name, version=bento_version, deleted=False)
                    .one()
                )
                # TODO:
                # if bento_obj.upload_status and bento_obj.upload_status.updated_at >
                # upload_status_pb.updated_at, update should be ignored
                bento_obj.upload_status = ProtoMessageToDict(upload_status_pb)
            except NoResultFound:
                raise BentoMLRepositoryException(
                    "Bento %s:%s is not found in repository" % bento_name, bento_version
                )

    def dangerously_delete(self, bento_name, bento_version):
        with create_session(self.sess_maker) as sess:
            try:
                bento_obj = (
                    sess.query(Bento)
                    .filter_by(name=bento_name, version=bento_version)
                    .one()
                )
                if not bento_obj.deleted:
                    raise BentoMLRepositoryException(
                        "Bento %s:%s has already been deleted" % bento_name,
                        bento_version,
                    )
                bento_obj.deleted = True
            except NoResultFound:
                raise BentoMLRepositoryException(
                    "Bento %s:%s is not found in repository" % bento_name, bento_version
                )

    def list(self, bento_name=None, offset=None, limit=None, filter_str=None):
        with create_session(self.sess_maker) as sess:
            query = sess.query(Bento)
            if limit:
                query.limit(limit)
            if offset:
                query.offset(offset)
            if filter_str:
                query.filter(Bento.name.contains(filter_str))
            if bento_name:
                query.filter_by(name=bento_name)

            return query.all()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: yatai_service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
import bentoml.proto.status_pb2 as status__pb2
import bentoml.proto.deployment_pb2 as deployment__pb2
import bentoml.proto.repository_pb2 as repository__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='yatai_service.proto',
  package='bentoml',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x13yatai_service.proto\x12\x07\x62\x65ntoml\x1a\x1bgoogle/protobuf/empty.proto\x1a\x0cstatus.proto\x1a\x10\x64\x65ployment.proto\x1a\x10repository.proto\"6\n\x13HealthCheckResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\"R\n\x1eGetYataiServiceVersionResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\x12\x0f\n\x07version\x18\x02 \x01(\t\"\x18\n\x05\x43hunk\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\x32\xd6\x07\n\x05Yatai\x12\x43\n\x0bHealthCheck\x12\x16.google.protobuf.Empty\x1a\x1c.bentoml.HealthCheckResponse\x12Y\n\x16GetYataiServiceVersion\x12\x16.google.protobuf.Empty\x1a\'.bentoml.GetYataiServiceVersionResponse\x12T\n\x0f\x41pplyDeployment\x12\x1f.bentoml.ApplyDeploymentRequest\x1a .bentoml.ApplyDeploymentResponse\x12W\n\x10\x44\x65leteDeployment\x12 .bentoml.DeleteDeploymentRequest\x1a!.bentoml.DeleteDeploymentResponse\x12N\n\rGetDeployment\x12\x1d.bentoml.GetDeploymentRequest\x1a\x1e.bentoml.GetDeploymentResponse\x12]\n\x12\x44\x65scribeDeployment\x12\".bentoml.DescribeDeploymentRequest\x1a#.bentoml.DescribeDeploymentResponse\x12T\n\x0fListDeployments\x12\x1f.bentoml.ListDeploymentsRequest\x1a .bentoml.ListDeploymentsResponse\x12?\n\x08\x41\x64\x64\x42\x65nto\x12\x18.bentoml.AddBentoRequest\x1a\x19.bentoml.AddBentoResponse\x12H\n\x0bUpdateBento\x12\x1b.bentoml.UpdateBentoRequest\x1a\x1c.bentoml.UpdateBentoResponse\x12?\n\x08GetBento\x12\x18.bentoml.GetBentoRequest\x1a\x19.bentoml.GetBentoResponse\x12i\n\x16\x44\x61ngerouslyDeleteBento\x12&.bentoml.DangerouslyDeleteBentoRequest\x1a\'.bentoml.DangerouslyDeleteBentoResponse\x12\x42\n\tListBento\x12\x19.bentoml.ListBentoRequest\x1a\x1a.bentoml.ListBentoResponseb\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,status__pb2.DESCRIPTOR,deployment__pb2.DESCRIPTOR,repository__pb2.DESCRIPTOR,])




_HEALTHCHECKRESPONSE = _descriptor.Descriptor(
  name='HealthCheckResponse',
  full_name='bentoml.HealthCheckResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.HealthCheckResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=111,
  serialized_end=165,
)


_GETYATAISERVICEVERSIONRESPONSE = _descriptor.Descriptor(
  name='GetYataiServiceVersionResponse',
  full_name='bentoml.GetYataiServiceVersionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.GetYataiServiceVersionResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='bentoml.GetYataiServiceVersionResponse.version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=167,
  serialized_end=249,
)


_CHUNK = _descriptor.Descriptor(
  name='Chunk',
  full_name='bentoml.Chunk',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='bentoml.Chunk.content', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=251,
  serialized_end=275,
)

_HEALTHCHECKRESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_GETYATAISERVICEVERSIONRESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
DESCRIPTOR.message_types_by_name['HealthCheckResponse'] = _HEALTHCHECKRESPONSE
DESCRIPTOR.message_types_by_name['GetYataiServiceVersionResponse'] = _GETYATAISERVICEVERSIONRESPONSE
DESCRIPTOR.message_types_by_name['Chunk'] = _CHUNK
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

HealthCheckResponse = _reflection.GeneratedProtocolMessageType('HealthCheckResponse', (_message.Message,), {
  'DESCRIPTOR' : _HEALTHCHECKRESPONSE,
  '__module__' : 'yatai_service_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.HealthCheckResponse)
  })
_sym_db.RegisterMessage(HealthCheckResponse)

GetYataiServiceVersionResponse = _reflection.GeneratedProtocolMessageType('GetYataiServiceVersionResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETYATAISERVICEVERSIONRESPONSE,
  '__module__' : 'yatai_service_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.GetYataiServiceVersionResponse)
  })
_sym_db.RegisterMessage(GetYataiServiceVersionResponse)

Chunk = _reflection.GeneratedProtocolMessageType('Chunk', (_message.Message,), {
  'DESCRIPTOR' : _CHUNK,
  '__module__' : 'yatai_service_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.Chunk)
  })
_sym_db.RegisterMessage(Chunk)



_YATAI = _descriptor.ServiceDescriptor(
  name='Yatai',
  full_name='bentoml.Yatai',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=278,
  serialized_end=1260,
  methods=[
  _descriptor.MethodDescriptor(
    name='HealthCheck',
    full_name='bentoml.Yatai.HealthCheck',
    index=0,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_HEALTHCHECKRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetYataiServiceVersion',
    full_name='bentoml.Yatai.GetYataiServiceVersion',
    index=1,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_GETYATAISERVICEVERSIONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ApplyDeployment',
    full_name='bentoml.Yatai.ApplyDeployment',
    index=2,
    containing_service=None,
    input_type=deployment__pb2._APPLYDEPLOYMENTREQUEST,
    output_type=deployment__pb2._APPLYDEPLOYMENTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteDeployment',
    full_name='bentoml.Yatai.DeleteDeployment',
    index=3,
    containing_service=None,
    input_type=deployment__pb2._DELETEDEPLOYMENTREQUEST,
    output_type=deployment__pb2._DELETEDEPLOYMENTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetDeployment',
    full_name='bentoml.Yatai.GetDeployment',
    index=4,
    containing_service=None,
    input_type=deployment__pb2._GETDEPLOYMENTREQUEST,
    output_type=deployment__pb2._GETDEPLOYMENTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DescribeDeployment',
    full_name='bentoml.Yatai.DescribeDeployment',
    index=5,
    containing_service=None,
    input_type=deployment__pb2._DESCRIBEDEPLOYMENTREQUEST,
    output_type=deployment__pb2._DESCRIBEDEPLOYMENTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListDeployments',
    full_name='bentoml.Yatai.ListDeployments',
    index=6,
    containing_service=None,
    input_type=deployment__pb2._LISTDEPLOYMENTSREQUEST,
    output_type=deployment__pb2._LISTDEPLOYMENTSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='AddBento',
    full_name='bentoml.Yatai.AddBento',
    index=7,
    containing_service=None,
    input_type=repository__pb2._ADDBENTOREQUEST,
    output_type=repository__pb2._ADDBENTORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateBento',
    full_name='bentoml.Yatai.UpdateBento',
    index=8,
    containing_service=None,
    input_type=repository__pb2._UPDATEBENTOREQUEST,
    output_type=repository__pb2._UPDATEBENTORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetBento',
    full_name='bentoml.Yatai.GetBento',
    index=9,
    containing_service=None,
    input_type=repository__pb2._GETBENTOREQUEST,
    output_type=repository__pb2._GETBENTORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DangerouslyDeleteBento',
    full_name='bentoml.Yatai.DangerouslyDeleteBento',
    index=10,
    containing_service=None,
    input_type=repository__pb2._DANGEROUSLYDELETEBENTOREQUEST,
    output_type=repository__pb2._DANGEROUSLYDELETEBENTORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListBento',
    full_name='bentoml.Yatai.ListBento',
    index=11,
    containing_service=None,
    input_type=repository__pb2._LISTBENTOREQUEST,
    output_type=repository__pb2._LISTBENTORESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_YATAI)

DESCRIPTOR.services_by_name['Yatai'] = _YATAI

# @@protoc_insertion_point(module_scope)
